/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ReflectionMapper.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.display;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class ReflectionMapper {
    private static Logger logger = Logger.getLogger(ReflectionMapper.class.getName());
    static Map s_getters = new HashMap();

    private static class Getter {
        public Method method;
        public String name;
    }

    private static void createGetters(Class obj) {
        try {
            List getters = new ArrayList();
            if(logger.isDebugEnabled()) {
                logger.debug("Class " + obj);
            }
            Method [] ms = obj.getMethods();
            for(int i = 1; i < ms.length; i++) {
                String name = ms[i].getName();
                if(logger.isDebugEnabled()) {
                    logger.debug("processing method " + name);
                }
                if(name.startsWith("get")) {
                    Getter g = new Getter();
                    g.method = ms[i];
                    g.name = name.substring(3).toUpperCase();
                    getters.add(g);
                    if(logger.isDebugEnabled()) {
                        logger.debug("Added Getter with name " + g.name);
                    }
                }
            }
            s_getters.put(obj, getters);
        } catch(SecurityException ex) {
        }
    }

    private static synchronized List getGetters(Class klass) {
        if(!s_getters.containsKey(klass)) {
            createGetters(klass);
        }
        return (List)s_getters.get(klass);
    }

    public static List map(List objs) {
        ArrayList res = new ArrayList(objs.size());
        for(Iterator it = objs.iterator(); it.hasNext();) {
            res.add(map(it.next()));
        }
        return res;
    }

    public static LightweightTemplateModel map(Object obj) {
        return map(new HashMapModel(), obj);
    }

    public static LightweightTemplateModel map(LightweightTemplateModel model, Object obj) {
        if(obj == null || model == null)
            return model;

        List getters = getGetters(obj.getClass());


        for(Iterator it = getters.iterator(); it.hasNext();) {
            try {
                Getter g = (Getter)it.next();
                Object value = g.method.invoke(obj, (Object[]) null);
                if(logger.isDebugEnabled()) {
                    logger.debug("Value: " + value);
                }
                model.setItem(g.name, value);
            } catch(IllegalAccessException ex) {
                ex.printStackTrace();
            } catch(InvocationTargetException ex) {
                ex.printStackTrace();
            }
        }
        return model;
    }
}
