/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SwitchFilter.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.display.filters;

import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.barracudamvc.contrib.dbroggisch.display.filters.dtd.Switch;


/**
 * This class executes the <code>filter</code>-method subsequently for all
 * ist child-filters. The first one not to return <b>null</b> or throwing
 * a <code>FilterException</code> wins.
 *
 * @author <a href="mailto:deets@web.de">Diez Roggisch</a>
 * @version 1.0
 */
public class SwitchFilter implements Filter {
    static Logger logger = Logger.getLogger(SwitchFilter.class.getName());
    Filter [] filters;


    public SwitchFilter() {
    }

    public SwitchFilter(Filter [] fs) {
        filters = fs;
    }

    public SwitchFilter(List fs) {
        this((Filter[])fs.toArray(new Filter[1]));
    }



    /* The method invokes the filter of all given filters
     * from first to last. The first one not returning null or throwing
     * an exception wins.
     *
     * @param value to be filtered
     * @return The first not-null value of the
     * @exception org.barracudamvc.contrib.dbroggisch.repopulation.FilterException <description>
     */
    public Object filter(Object value, FilterContext context)
        throws FilterException
    {
        Object res = null;
        for(int i = 0; i < filters.length; i++) {
            try {
                res = filters[i].filter(value, context);
                if(logger.isDebugEnabled()) {
                    logger.debug("Switch result: " + res);
                }
            } catch(FilterException ex) {
            }
            if(res != null) {
                break;
            }
        }
        return res;
    }

    public Filter configure(Object obj)
        throws FilterException {
        Switch fw = (Switch)obj;
        List childs = FilterFactory.accumulateChilds(fw);
        filters = new Filter[childs.size()];
        int i = 0;
        for(Iterator it = childs.iterator(); it.hasNext();) {
            Object decl = it.next();
            filters[i] = FilterFactory.createFilter(decl.getClass());
            filters[i++].configure(decl);
        }
        return this;
    }


}
