/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SelectFormValidator.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.repopulation;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.ItemMap;
import org.barracudamvc.core.comp.ListModel;
import org.barracudamvc.core.comp.ListSelectionModel;
import org.barracudamvc.core.forms.DefaultFormValidator;
import org.barracudamvc.core.forms.FormElement;
import org.barracudamvc.core.forms.ValidationException;

public class SelectFormValidator extends DefaultFormValidator {
    protected static final Logger logger = Logger.getLogger(SelectFormValidator.class.getName());

    int selMode = -1;
    String ignore;

    public void setSelectionMode(int mode) {
        selMode = mode == ListSelectionModel.MULTIPLE_INTERVAL_SELECTION ?
            mode : mode == ListSelectionModel.SINGLE_INTERVAL_SELECTION ?
            mode : mode == ListSelectionModel.SINGLE_SELECTION ?
            mode : selMode;
    }

    public SelectFormValidator(String ignore) {
        //super();
        this.ignore = ignore;
    }

    public SelectFormValidator(String ignore, String message) {
        super(message);
        this.ignore = ignore;
    }

    public SelectFormValidator() {
        //super();
    }

    public SelectFormValidator(int selMode) {
        //super();
        setSelectionMode(selMode);
    }

    public SelectFormValidator(int selMode, String message) {
        super(message);
        setSelectionMode(selMode);
    }

    public SelectFormValidator(int selMode, String ignore, String message) {
        super(message);
        this.ignore = ignore;
        setSelectionMode(selMode);
    }


    public int getSelectionMode() {
        return selMode;
    }

    public void setIgnore(String val) {
        ignore = val;
    }

    public String getIgnore() {
        return ignore;
    }


    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        if(logger.isDebugEnabled()) {
            logger.debug("Begin validation.");
        }
        if(!(element instanceof SelectFormElement)) {
            logger.error("Not a SelectFormElement" + element);
            throw this.generateException(element, deferExceptions, "Element must be SelectFormElement");
        }

        boolean allContained = true;
        SelectFormElement selEl = (SelectFormElement)element;
        ListModel lm = selEl.getListModel();
        List selected = new ArrayList();
        Object origVal = selEl.getOrigVal();

        // Get the list of selected values - as a list
        if(origVal != null) {
            if(logger.isDebugEnabled()) {
                logger.debug("OrigVal class is:" + origVal.getClass());
            }
            if(origVal instanceof List) {
                selected = (List)origVal;
            } else {
                selected.add(origVal);
            }
        }

        if(ignore != null) {
            selected.remove(ignore);
        }


        // Get the keys/values from the ListModel - as Strings!
        List values = new ArrayList(lm.getSize());
        for(int i = 0; i < lm.getSize(); i++) {
            String key;
            String value;
            Object item = lm.getItemAt(i);
            if(item instanceof ItemMap) {
                key = ((ItemMap)item).getKey().toString();
                value = ((ItemMap)item).getValue().toString();
            } else {
                value = key = item.toString();
            }
            if(!(ignore != null && ignore.equals(value)))
                values.add(key);
        }
        // Every selected value _must_ be among the allowed values. Otherwise - error!
        for(int i = 0;i < selected.size(); i++) {
            if(!values.contains(selected.get(i))) {
                allContained = false;
                break;
            }
        }
        if(!allContained) {
            if(logger.isDebugEnabled()) {
                logger.debug("Not all selected options are among the permitted values.");
            }
            throw this.generateException(element, deferExceptions, getErrorMessage() != null ? getErrorMessage() : "Not all selected options are among the permitted values.");
        }
        // Do we have a value at all? If not,
        // throw an error.
        if(ignore != null && selected.size() == 0) {
            if(logger.isDebugEnabled()) {
                logger.debug("The selected value was the ignored one.");
            }
            throw this.generateException(element, deferExceptions, getErrorMessage() != null ? getErrorMessage() :  "The selected value was the ignored one.");
        }
        if(selMode != -1 && SelectFormElement.calcSelectionMode(values, selected) != selMode) {
            if(logger.isDebugEnabled()) {
                logger.debug("The selection mode differs.");
            }
            throw this.generateException(element, deferExceptions, getErrorMessage() != null ? getErrorMessage() :  "The selection mode differs.");
        }
    }
}
