/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ExampleFormMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.examples.model.forms;

import org.barracudamvc.contrib.dbroggisch.repopulation.*;
import org.barracudamvc.core.forms.*;
import org.barracudamvc.core.comp.*;
import org.barracudamvc.core.forms.validators.*;

public class ExampleFormMap extends RepopulationFormMap {

    public final static String MODEL_NAME = "ExampleModel";
    public final static String GROUP = "GROUP";
    public final static String CHECKBOX = "CHECKBOX";
    public final static String SELECT = "SELECT";
    public final static String TEXTAREA = "TEXTAREA";
    public final static String SIMPLECHECKBOX = "SIMPLECHECKBOX";
    public ExampleFormMap() {
        setName(MODEL_NAME);
        // This element will create its own child templatemodel with the name
        // MODEL_NAME + "_" + GROUP
        // The choices will be represented by radiobuttons due to the allowMultipleValues property
        // beeing set to false.
        defineElement(new GroupFormElement(
                                           new TestList(),
                                           GROUP,
                                           FormType.INTEGER,
                                           null,
                                           new And(new SelectFormValidator(), new NotNullValidator()),
                                           false));

        // This element is similar to the preceeding one, with the exception that it will be rendered
        // useing CheckBoxes - it allows for multiple values.
        defineElement(new GroupFormElement(
                                           new TestList(),
                                           CHECKBOX,
                                           FormType.INTEGER,
                                           null,
                                           new And(new SelectFormValidator(), new NotNullValidator()),
                                           true));

        defineElement(new SelectFormElement(
                                            new TestList(),
                                            SELECT,
                                            FormType.INTEGER,
                                            null,
                                            new SelectFormValidator("Element no. " + 0, "You're not allowed to select the first element!"),
                                            false));

        defineElement(new TextAreaFormElement(
                                              TEXTAREA,
                                              null,
                                              null));

        defineElement(new CheckboxFormElement(SIMPLECHECKBOX,
                                              FormType.BOOLEAN));

    }

    public static class TestList extends DefaultListModel {

        public TestList() {
            for(int i = 0; i < 5; i++) {
                add(new DefaultItemMap("" + i, "Element no. " + i));
            }
        }
    }
}
