/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DefaultConfig.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.data;

import java.lang.reflect.Constructor;

import org.apache.log4j.Logger;
import org.barracudamvc.core.event.ViewEvent;

public abstract class DefaultConfig implements Config {

    protected static Logger logger = Logger.getLogger(DefaultConfig.class.getName());

    protected Class viewEvent;
    protected String xmlFormName;
    protected String formName;
    protected String MAP = "MAP";
    protected String OID = "oid";

    public DefaultConfig(Class viewEvent, String xmlFormName, String formName) {
        this.viewEvent = viewEvent;
        this.xmlFormName = xmlFormName;
        this.formName = formName;
    }

    public ViewEvent getViewEvent() {
        try {
            Class[] args = new Class[0];
            Object[] param = new Object[0];
            Constructor c = viewEvent.getConstructor(args);
            return (ViewEvent)c.newInstance(param);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex) ;
            return null;
        }
    }

    public String getXmlFormName() {
        return xmlFormName;
    }

    public String getFormName() {
        return formName;
    }

    public String getMapName() {
        return MAP;
    }

    public String getOidName() {
        return OID;
    }

    public abstract DataObject getDataObject(String oid) throws DataObjectException;

}
