/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: RegexValidator.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.data;

import gnu.regexp.RE;

import org.barracudamvc.core.forms.DefaultFormValidator;
import org.barracudamvc.core.forms.FormElement;
import org.barracudamvc.core.forms.FormType;
import org.barracudamvc.core.forms.ValidationException;

/** This validator ensures that a value matches a given regexp.
 * For compatibility with JDK 1.3, all references to java.util.regex are replaced by gnu.regexp
 */
public class RegexValidator extends DefaultFormValidator {

     // JDK 1.4: uncomment the next line and comment the following line out
    //protected Pattern re;
     protected RE re;

    /**
     * Public constructor.
     *
     * @param re Pattern to be matched
     */
     // JDK 1.4: uncomment the next line and comment the following line out
    //public RegexValidator(Pattern re ) {
    public RegexValidator(RE re ) {
        super();
        this.re = re;
    }

    /**
     * Public constructor.
     *
     * @param re Pattern to be matched
     * @param ierrmsg the message associated with this error
     */
     // JDK 1.4: uncomment the next line and comment the following line out
    //public RegexValidator(Pattern re, String ierrmsg) {
    public RegexValidator(RE re, String ierrmsg) {
        super(ierrmsg);
        this.re = re;
    }

    /**
     * Validate a FormElement to make sure a given pattern is matched
     * Validation is only supported for FormType.STRING
     *
     * @param element the form element to be validated
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        if (this.isNull(val, element))
            return;

        if (element==null) throw new ValidationException(val, "Object val:"+val+" is associated with a null FormElement");
        if (!element.getType().equals(FormType.STRING)) throw new ValidationException(val, "Unsupported validation: "+val+" is not of FormType.String and cannot be validated by this validator");

        String s = val.toString();
     // JDK 1.4: uncomment the next line and comment the following line out
        //if (!re.matcher(s).matches()) {
        if (!re.isMatch(s)) {
     // JDK 1.4: uncomment the next line and comment the following line out
            //throw this.generateException(element, deferExceptions, "{" + val + "}" + " does not match Pattern " + re.pattern());
            throw this.generateException(element, deferExceptions, "{" + val + "}" + " does not match Pattern " + re.toString());
        }
    }

}
