package org.barracudamvc.contrib.sam.data;

import org.apache.log4j.Logger;
import org.barracudamvc.contrib.dbroggisch.repopulation.RepopulationElement;
import org.barracudamvc.core.comp.BAction;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.event.ControlEvent;
import org.barracudamvc.core.forms.DefaultFormElement;
import org.barracudamvc.core.forms.FormElement;
import org.barracudamvc.core.forms.FormType;

/**
 * @author Stefan Armbruster (2003-11-16)
 *
 * 
 */
public class UploadLinkElement extends DefaultFormElement implements RepopulationElement{
	
	private FormElement idElement;

	protected static Logger logger = Logger.getLogger(UploadLinkElement.class);

	/**
	 * fieldname to get dataobjects identiry
	 */
	private String idField;

	/**
	 * this field's name, ends on "XX_link"
	 */
	private String field;

	/**
	 * event to be called on click
	 */
	private String linkEvent;

	/**
	 * dataobject's field name for filename
	 */
	private String filename_field;

	/**
	 * dataobject's field name for content type
	 */
	private String cnttype_field;

	/**
	 * @param name
	 * @param cnttype_field
	 * @param filename_field
	 */
	public UploadLinkElement(String name, String field, String cnttype_field, String filename_field, String linkEvent, String idField) {
		super(name, FormType.STRING, null);
		this.field = field;
		this.cnttype_field = cnttype_field;
		this.filename_field = filename_field;
		this.linkEvent = linkEvent;
		this.idField = idField;
	}

	/**
	 * @return Name of the field containing the content type
	 */
	public String getContentTypeField() {
		return cnttype_field;
	}

	/**
	 * @return Name of the field containing the filename
	 */
	public String getFileNameField() {
		return filename_field;
	}
	
	protected ControlEvent getLinkEvent() {
		try {
			Class c = Class.forName(linkEvent);
			ControlEvent event = (ControlEvent) c.newInstance();
			return event;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new RuntimeException(e);
		}
	}

	/**
	 * return a @see BAction with parameters: idField, cnttype_field, filename_field
	 */
	public Object render(ViewContext context) {
		logger.debug("render called");
		if (idElement == null) {
			logger.warn("no idElement set, discarding BAction element, returning null instead");
			return null;
		}
		Object origVal = idElement.getOrigVal();
		if (origVal==null) {
			logger.info("origval is null");
			return null;
		}
		BAction ret = new BAction(getLinkEvent());
		ret.setParam(idField, origVal.toString());
		ret.setParam("cnttype_field", cnttype_field);
		ret.setParam("filename_field", filename_field);
		return ret;
	}

	/**
	 * @return Name of ID field
	 */
	public String getIdField() {
		return idField;
	}

	/**
	 * @param idEle
	 */
	public void setIdFormElement(FormElement idEle) {
		//logger.debug("setting IdElement to " + idEle.getKey() + "|" + idEle.getName());
		this.idElement = idEle;
	}

	/**
	 * @return this field's name, ends on "XX_link"
	 */
	public String getField() {
		return field;
	}

}
