/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ReflectionModel.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.models;

/** provide a model to retrieve data by using the reflection API.
* for each key, a method get<key> is called without parameters, the return value is passed through*/

import java.lang.reflect.Method;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.AbstractTemplateModel;

public class ReflectionModel extends AbstractTemplateModel {

    protected static Logger logger = Logger.getLogger(ReflectionModel.class);
    protected String name;
    protected Object obj;

    public ReflectionModel(String name) {
        this.name = name;
    }

    public ReflectionModel(String name, Object obj) {
        this.name = name;
        setObject(obj);
    }

    public void setObject(Object obj) {
        this.obj = obj;
    }

    public String getName() {
        return name;
    }

    public Object getItem(String key) {
        try {
            Method m = obj.getClass().getMethod("get" + key, (Class[]) null);
            return m.invoke(obj, (Object[]) null);

        } catch (Exception ex) {
            logger.error(ex.getMessage() );
            return ex.getMessage();
        }
    }
}
