/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: HTMLScriptResourceRenderer.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.comp.renderer.html;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BScriptResource;
import org.barracudamvc.core.comp.NoSuitableRendererException;
import org.barracudamvc.core.comp.RenderException;
import org.barracudamvc.core.comp.View;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.renderer.DOMComponentRenderer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLHeadElement;
import org.w3c.dom.html.HTMLScriptElement;

/**
 * This class handles the default rendering script references into an HTML view.
 */
public class HTMLScriptResourceRenderer extends DOMComponentRenderer {

    protected static final Logger logger = Logger.getLogger(HTMLScriptResourceRenderer.class.getName());


    /**
     *
     */
    public void renderComponent(BComponent comp, View view, ViewContext vc) throws RenderException {
        //unlike other components, this one really doesn't care what we're bound
        //to...we do need to make sure the component is a script component
        if (!(comp instanceof BScriptResource)) throw new NoSuitableRendererException("This renderer can only render BScriptResource components");        

        //show what we're doing
        showNodeInterfaces(view, logger);
        
        //now allow the parent class to do anything it needs to
        super.renderComponent(comp, view, vc);

        BScriptResource bsr = (BScriptResource) comp;
        String src = bsr.getSrc();
        
        //Shortcircuit rendering if no src to set
        if (src == null) {
        	logger.warn("Null src, skipping rendering.");
        	return;
        }
        
        Node node = view.getNode();
        
        //find the root
        Document doc = node.getOwnerDocument();
        Element elRoot = doc.getDocumentElement();

        //now see if there is a head element
        Element elHead = null;
        Node child = elRoot.getFirstChild();
        while (child!=null) {
            if (child instanceof HTMLHeadElement) {
                elHead = (HTMLHeadElement) child;
                break;
            }
            child = child.getNextSibling();
        }

        //if not, create one and add it in
        if (elHead==null) {
            elHead = doc.createElement("HEAD");
            elRoot.insertBefore(elHead, elRoot.getFirstChild());
        }

        //now see if we have script elements that match already in place
        //(if so, we don't need to do anything)
        HTMLScriptElement elScript = null;
        child = elHead.getFirstChild();
        while (child!=null) {
            if (child instanceof HTMLScriptElement) {
                HTMLScriptElement scel = (HTMLScriptElement) child;
                if (src.equals(scel.getSrc())) {
                    elScript = scel;
                    break;
                }
            }
            child = child.getNextSibling();
        }

        //if we don't, create one
        if (elScript==null) {
            elScript = (HTMLScriptElement) doc.createElement("SCRIPT");
            elHead.appendChild(elScript);
            elScript.setType("text/javascript");
            elScript.setSrc(src);
        }
    }

}