/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SampleViewHandler.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.examples.ex4;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.event.BaseEventListener;
import org.barracudamvc.core.event.DefaultBaseEventListener;
import org.barracudamvc.core.event.DefaultListenerFactory;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.ViewEventContext;


/**
 * A sample view event handler
 */
public class SampleViewHandler extends DefaultListenerFactory {

    public static final String HANDLED_BY = "HandledBy";

    //define the necessary factory methods
    public BaseEventListener getInstance() {
        return new RenderHandler();
    }            
    public String getListenerID() {
        return getID(RenderHandler.class);
    }

    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderHandler - handle the request to render the test 
     * screen
     */
    class RenderHandler extends DefaultBaseEventListener {
        public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
            HttpServletResponse resp = context.getResponse();

            //csc_061202.1 - added    
            //set the caching hdrs (allow this page to be cached by browser)
            resp.setHeader("Cache-Control","max-age=0");
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());

            //build the response
            resp.setContentType("text/html");
            PrintWriter out = resp.getWriter();
            
            //header            
            String title = "Simple Deployment Descriptor Test Screen";
            out.println ("<html>");
            out.println ("  <head>");
            out.println ("    <title>"+title+"</title>");
            out.println ("  </head>");
            out.println ("  <body>");
            out.println ("    <font face=\"Arial\">");

            //body
            out.println ("      <p>"+title);
            out.println ("      <p>Handled by:"+context.getState(HANDLED_BY));
            
            //footer
            out.println ("    </font>");
            out.println ("  </body>");
            out.println ("</html>");
        }
    }
}

