package org.barracudamvc.examples.xmlc;

import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BTemplate;
import org.barracudamvc.core.comp.DefaultTemplateView;
import org.barracudamvc.core.comp.TemplateModel;
import org.barracudamvc.core.comp.TemplateView;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.helper.ComponentGateway;
import org.barracudamvc.core.util.dom.DOMWriter;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.core.util.dom.DefaultDOMWriter;
import org.barracudamvc.core.view.ViewCapabilities;
import org.w3c.dom.Document;

/**
 * Check to see who Barracuda thinks we are
 */
public class BrowserCheck extends ComponentGateway {

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the localized DOM template
//csc_061702.1        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(BrowserCheckHTML.class);
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(BrowserCheckHTML.class, vc.getViewCapabilities().getClientLocale());    //csc_061702.1

        //create a template component and bind it to the views
        TemplateModel tm = new LocalTemplateModel(req);
        TemplateView tv = new DefaultTemplateView(page.getDocumentElement());
        BTemplate templateComp = new BTemplate(tm);
        templateComp.setView(tv);
        root.addChild(templateComp);

        //return the page
        return page;
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(true);
    }

    //--------------- TemplateModel --------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
    
        HttpServletRequest req = null;
        
        public LocalTemplateModel(HttpServletRequest ireq) {
            req = ireq;
        }
        
        //register the model by name
        public String getName() {return "BrowserCheck";}
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            ViewCapabilities vcap = vc.getViewCapabilities();
            if (key.equals("FormatType")) {
                return vcap.getFormatType();
            } else if (key.equals("ClientType")) {
                return vcap.getClientType();
            } else if (key.equals("ScriptingType")) {
                return vcap.getScriptingType();
            } else if (key.equals("User-Agent")) {
                return enumToString(req.getHeaders("User-Agent"));
            } else if (key.equals("Accept")) {
                return enumToString(req.getHeaders("Accept"));
            } else {
                return super.getItem(key);
            }
        }
        
        String enumToString(Enumeration enumer) {
            StringBuffer sb = new StringBuffer(100);
            String sep = "";
            while (enumer.hasMoreElements()) {
                sb.append(sep+enumer.nextElement());
                sep = ",<br>";
            }
            return sb.toString();
        }
    }
}
