package org.barracudamvc.examples.xmlc;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BLink;
import org.barracudamvc.core.comp.BTemplate;
import org.barracudamvc.core.comp.BText;
import org.barracudamvc.core.comp.DefaultTemplateView;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.helper.ComponentGateway;
import org.barracudamvc.core.util.dom.DOMWriter;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.core.util.dom.DefaultDOMWriter;
import org.w3c.dom.Document;

/**
 * Test BText ability to return encoded text, as well as markup 
 * Created to test the implementation of csc_092701.1
 */
public class CompEx7 extends ComponentGateway {

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(CompEx7HTML.class);

        //create a template component and bind it to the views
        BTemplate templateComp = new BTemplate(new LocalTemplateModel(req));
        templateComp.setView(new DefaultTemplateView(page.getDocumentElement()));
        root.addChild(templateComp);

        //return the page
        return page;
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(true);
    }

    //--------------- TemplateModel --------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
    
        HttpServletRequest req = null;
        
        public LocalTemplateModel(HttpServletRequest ireq) {
            req = ireq;
        }
        
        //register the model by name
        public String getName() {return "Test1";}
        
        //provide items by key
        public Object getItem(String key) {
            if (key.equals("NormalText1")) {
                return "Text1: <strong>bold</strong>";
            } else if (key.equals("NormalText2")) {
                return new BLink("Link1: A link with <em>zing1</em>", "blah99.html");
            } else if (key.equals("MarkupText1")) {
                BText btext = new BText("Text2: <strong>bold</strong>");
                btext.setAllowMarkupInText(true);
                return btext;
            } else if (key.equals("MarkupText2")) {
                BLink blink = new BLink("Link2: A link with <em>zing2</em>", "blah99.html");
                blink.setAllowMarkupInText(true);
                return blink;
            } else {
                return super.getItem(key);
            }
        }
    }
}
