package org.barracudamvc.testbed.workbench.jdbc;

import java.beans.PropertyVetoException;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

/**
 * Database - this is really a legacy class; we should be using JDBC 2.0 and getting 
 * a DataSource, but for now we'll leave it like it is, since after all it already 
 * works ;-)
 *
 * @author Christian Cryder (csc)
 * @version 1.0
 */
public class Database {

    private Connection conn = null;
    private String jdbcDriverName = "";
    private String jdbcServerURL = "";
    private    String dbUserName = "";
    private    String dbPassword = "";
    private Properties props = new Properties();
    private Driver theJDBCDriver = null;
    private int commitLevel = -1;
    private    String userName = "";

    /**
     * The public contructor for Database
     *
     * @param ijdbcDriverName Jdbc Driver Name
     * @param ijdbcServerURL Jdbc Server URL
     * @param idbUserName Database User Name
     * @param idbPassword Database password
     */
    public Database (String ijdbcDriverName, String ijdbcServerURL, String idbUserName, String idbPassword)    {
        jdbcDriverName = ijdbcDriverName;
        jdbcServerURL = ijdbcServerURL;
        dbUserName = idbUserName;
        dbPassword = idbPassword;
    }

    /**
     * connectServer
     *
     * @exception PropertyVetoException
     */
    public void connectServer() throws java.beans.PropertyVetoException    {
        // set the timeout so that the server closes if    fail to    connect
        DriverManager.setLoginTimeout(20);

        //Make the connection to the database
        try {
            // create a    specific driver    for    use    with JDBC
            theJDBCDriver =    (Driver)Class.forName(jdbcDriverName).newInstance();

            // connect to the database using the JDBC driver
            props.put("user",dbUserName);
            props.put("password",dbPassword);
            conn = makeConnection(theJDBCDriver, jdbcServerURL, props);
            setAutoCommit(true);
        } catch (SQLException se) {
            String message = null;
            message = se.toString();
            throw new java.beans.PropertyVetoException("Error Connecting to Database: "+message,null);
        } catch (Exception e) {
            throw new java.beans.PropertyVetoException("Error Connecting to Database: "+e,null);
        }
    }

    /**
     * makeConnection
     *
     * @param ijdbcDriverName Jdbc Driver Name
     * @param ijdbcServerURL Jdbc Server URL
     * @param props Properties
     * @return Connection
     * @exception SQLException
     */
    private static synchronized Connection makeConnection (Driver theJDBCDriver, String jdbcServerURL, Properties props) throws SQLException {
        return theJDBCDriver.connect(jdbcServerURL, props);
    }

    /**
     * disconnectServer
     *
     * @exception PropertyVetoException
     */
    public void disconnectServer() throws java.beans.PropertyVetoException {
        //close the connection. If we do get an error, things are really hosed, so
        //set the conn = null
        try    {
            if (conn!=null) conn.close();
        } catch (Exception e) {
            throw new java.beans.PropertyVetoException("Error Disconnecting from Database:"+e,null);
        } finally {
            conn = null;
        }
    }

    /**
     * disconnectServer
     *
     * @return True if still connect to the database
     */
    public boolean isConnected() {
        return conn!=null;
    }

    /**
     * disconnectServer
     *
     * @param s String to be set in the prepare statement
     * @return PreparedStatement
     * @exception SQLException
     */
    public PreparedStatement prepareStatement(String s) throws SQLException {
        for (;;) {
            try {
                //if the connection has dropped for some reason, restablish
                //we will continue doing this until the connection is established
                while (!isConnected()) {
                    try {
                        connectServer();
                    } catch (java.beans.PropertyVetoException pve) {
                        System.out.println ("Server not responding...");
                        try {Thread.sleep(200);}
                        catch (InterruptedException ie) {}
                    } finally {
                        if (isConnected()) System.out.println ("Connection reestablished...");
                    }
                }

                //return the connection
                return conn.prepareStatement(s);
            } catch (SQLException se) {
                String err = se.getSQLState();
                //see if we're having trouble talking to the server, and if so, disconnect.
                //This will cause the server to automatically reconnect on the next pass through
                //the loop
                if (err.equals("08S01")) {
                    try {disconnectServer();}
                    catch (java.beans.PropertyVetoException pve) {}
                } else {
                    throw se;
                }
            }
        }
    }

    /**
     * createStatement
     *
     * @return Statement
     * @exception SQLException
     */
    public Statement createStatement() throws SQLException {
        for (;;) {
            try {
                //if the connection has dropped for some reason, restablish
                //we will continue doing this until the connection is established
                while (!isConnected()) {
                    try {connectServer();}
                    catch (java.beans.PropertyVetoException pve) {
                        System.out.println ("Server not responding...");
                        try {Thread.sleep(200);}
                        catch (InterruptedException ie) {}
                    } finally {
                        if (isConnected()) System.out.println ("Connection reestablished...");
                    }
                }

                //return the connection
                return conn.createStatement();
            } catch (SQLException se) {
                String err = se.getSQLState();
                //see if we're having trouble talking to the server, and if so, disconnect.
                //This will cause the server to automatically reconnect on the next pass through
                //the loop
                if (err.equals("08S01")) {
                    try {disconnectServer();}
                    catch (java.beans.PropertyVetoException pve) {}
                } else {
                    throw se;
                }
            }
        }
    }

    /**
     * getConnection
     *
     * @return Connection
     * @exception PropertyVetoException
     */
    public Connection getConnection() throws java.beans.PropertyVetoException {
        return getConnection(false);
    }

    /**
     * getConnection
     *
     * @param verify Verify
     * @return Connection
     * @exception PropertyVetoException
     */
    public Connection getConnection(boolean verify) throws java.beans.PropertyVetoException {
        //only verify the connection if specifically asked to do so (this will renegotiate the
        //connection if necessary)
        if (verify) {
            try {verifyConnection();}
            catch (SQLException se) {throw new java.beans.PropertyVetoException("Error getting the Connection to Database:"+se,null);}
        }

        //return the connection
        return conn;
    }

    /**
     * setConnection
     *
     * @param c Connection to be set
     * @see Connection
     */
    public void setConnection (Connection c) {
        conn = c;
    }

    /**
     * verifyConnection
     *
     * @exception SQLException
     */
    public void verifyConnection() throws SQLException {
        verifyConnection("select count(*) from metadata");    //we really ought to try and come up with a generic chunk of SQL here...something that's not specific to either database or tables
    }
    
    public void verifyConnection(String s) throws SQLException {
        this.prepareStatement(s).close();
    }

    /**
     * resetConnection
     */
    public void resetConnection() {
        conn = null;
    }

    /**
     * commit
     *
     * @exception SQLException
     */
    public void commit() throws SQLException {
        int cl = getCommitLevel();
           if (cl>0) return;

           if (!getAutoCommit()) {
               conn.commit();
               resetCommitLevel();        //just to make sure
           }
    }

    /**
     * rollback
     *
     * @exception SQLException
     */
    public void rollback() throws SQLException {
        int cl = getCommitLevel();
           if (cl>0) return;

           if (!getAutoCommit()) {
               conn.rollback();
               resetCommitLevel();        //just to make sure
           }
    }

    /**
     * resetCommitLevel
     */
    public void resetCommitLevel() {
        //This is kind of a safety valve...reset it just to make sure
        commitLevel = -1;
    }

    /**
     * getCommitLevel
     */
    public int getCommitLevel() {
        //return the current commitLevel
        return commitLevel;
    }

    /**
     * setAutoCommit
     *
     * @param val True if commited
     */
    public void setAutoCommit(boolean val) {
        //if the level is < 0, we know we're at the base...set the value
        if (commitLevel<0) {
            try {conn.setAutoCommit(val);}
            catch (SQLException se) {
                System.out.println ("Error setting AutoCommit...");
                Throwable th = new Throwable();
                th.printStackTrace();
            }
        }
        
        //incr our commitLevel...this is to handle nested data objects. If
        //UserData contains a UserAccessDataSet object, and each of these
        //has rollback and commit potential, we need to be careful when we
        //actually do the deed. If UserAccessDataSet exists all on its own
        //and we hit an error, the rollback is appropriate immediately.
        //If however, the UserAccessDataSet exists in the context of a UserData
        //object, we need to defer on the rollback, pass the SQLException back
        //on up and let the UserData object handle the rollback.
        if (val==false) commitLevel++;                //turning it off
        else if (commitLevel>-1) commitLevel--;        //turning it back on
    }

    /**
     * getAutoCommit
     *
     * @return True if commited
     */
    public boolean getAutoCommit() {
        try {return conn.getAutoCommit();}
        catch (SQLException se) {
            System.out.println ("Error getting AutoCommit...");
            Throwable th = new Throwable();
            th.printStackTrace();
        }
        return false;
    }


    /**
     * getUserName
     *
     * @return User name
     */
    public String getUserName() {
        return userName;
    }

    /**
     * setUserName
     *
     * @param s User name to be set
     */
    public void setUserName(String s) {
        userName = s;
    }
}
