package org.barracudamvc.testbed.workbench.stress;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.net.MalformedURLException;
import java.util.*;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.barracudamvc.plankton.*;
import org.barracudamvc.plankton.http.HttpConverter;
import org.barracudamvc.plankton.http.HttpRequester;
import org.barracudamvc.testbed.workbench.util.gui.Containers;
import org.barracudamvc.testbed.workbench.util.gui.Cursors;
import org.barracudamvc.testbed.workbench.util.gui.DefaultComponentListener;

/**
 * URL Stress Tester
 */
public class URLStressTester extends JPanel {

    //public vars
    public static String DEFAULT_URL = "http://localhost:8080/Barracuda/CompTest10?$usebi";
    public static String DEFAULT_NUMBER_OF_THREADS = "1";
    public static String DEFAULT_NUMBER_OF_TIMES_TO_LOOP = "10";

    //debug vars
    public static int showDebug = 0;

    //test variables
    private int activeThreads = 0;
//    private int threadCntr = 0;
    private long startTime = 0;
    private long endTime = 0;
    private long totalTime = 0;
    private boolean readySetGo = false;
    private boolean logToConsole = true;
    private int wrapOutput = 80;
    private boolean showDetails = false;
    private boolean doGet = true;
    private boolean isMozilla = true;
//    private boolean isIE = false;
    private Map thElapsed = null;
    
    //objects
    private boolean isInitialized = false;

    //buttons & controls
    //...URL panel
    private JTextField jtfURL = null;
    private JButton jbRunTest = null;
    private JButton jbAvgTest = null;
    private JButton jbAddToReq = null;
    //...Main panel
    private JSplitPane jspMaster = null;
    //...Requests panel
    private JPanel jpRequests = null;
    //...Output panel
    private JPanel jpOutput = null;
    //...Footer panel
    private JTextField jtfNoOfThreads = null;
    private JTextField jtfNoOfTimes = null;
    private JRadioButton jrbIE = null;
    private JRadioButton jrbMZ = null;
    private JRadioButton jrbGet = null;
    private JRadioButton jrbPost = null;
    private JTextField jtfLatency = null;
    private JTextField jtfWrapOutput = null;
    private JCheckBox jcbShowDetails = null;

    
    /**
     * Public constructor
     */
    public URLStressTester() {
        //set up a listener so when this guy is exposed we hear about it
        this.addComponentListener(new LocalComponentListener());
    }
    
    //============================================================
    //        Model / Controller info for the Master View
    //============================================================
    /**
     * This method is responsible for laying out the
     * UI components. 
     *
     * @throws Exception
     */
    private void initViews() throws Exception {

        //general
        this.setLayout(new GridBagLayout());
        LocalActionAdapter actionAdapter = new LocalActionAdapter();
        Insets buttonInsets = new Insets(0, 5, 0, 5);

        //URL Panel
        JPanel jpURL = new JPanel();
        jpURL.setLayout(new GridBagLayout());
        this.add(jpURL, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
        //...URL Label
        JLabel jlbURL = new JLabel("URL:");
        jpURL.add(jlbURL, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        //...URL Textfield
        jtfURL = new JTextField(DEFAULT_URL);
        jpURL.add(jtfURL, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 1.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 5), 0, 0));
        //...Run Test
        jbRunTest = new JButton("Run Test");
        jbRunTest.setToolTipText("Run the test");
        jbRunTest.setMargin(buttonInsets);
        jbRunTest.addActionListener(actionAdapter);
        jpURL.add(jbRunTest, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
        //...Avg Test
        jbAvgTest = new JButton("Avg Test");
        jbAvgTest.setToolTipText("Run the test 10x and average the results");
        jbAvgTest.setMargin(buttonInsets);
        jbAvgTest.addActionListener(actionAdapter);
        jpURL.add(jbAvgTest, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
        //...Add to List
        jbAddToReq = new JButton("+List");
        jbAddToReq.setMargin(buttonInsets);
        jbAddToReq.addActionListener(actionAdapter);
        jpURL.add(jbAddToReq, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));


        
        //Main Panel (Splitter)
        jspMaster = new JSplitPane();
        jspMaster.setOneTouchExpandable(true);
        this.add(jspMaster, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 1.0, 
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 5, 5), 0, 0));
        
        //...Requests Panel
        jpRequests = new JPanel();
        jpRequests.setLayout(new GridBagLayout());
        jspMaster.add(jpRequests, JSplitPane.LEFT);
        //......Todo
        jpRequests.add(new JLabel("todo"), new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 1.0, 
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
        
        //...Output Panel
        jpOutput = new JPanel();
        jpOutput.setLayout(new GridBagLayout());
        JScrollPane jspOutput = new JScrollPane(jpOutput);
        jspMaster.add(jspOutput, JSplitPane.RIGHT);
        //......Todo
//        jpOutput.add(new JLabel("todo"), new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 1.0, 
//            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
        
        jspMaster.setDividerLocation(0);        
        
        
        //Footer Panel
        JPanel jpFooter = new JPanel();
        jpFooter.setLayout(new GridBagLayout());
        this.add(jpFooter, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 2, 5), 0, 0));
        //...filler
        jpFooter.add(new JLabel(" "), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 1.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        //...Browser Type
        jpFooter.add(new JLabel("Browser:"), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 15, 0, 0), 0, 0));
        ButtonGroup bgbr = new ButtonGroup();
        jrbMZ = new JRadioButton("Mozilla");
        bgbr.add(jrbMZ);
        jrbMZ.setSelected(true);
        jpFooter.add(jrbMZ, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        jrbIE = new JRadioButton("IE");
        bgbr.add(jrbIE);
        jpFooter.add(jrbIE, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        //...Get/Post
        jpFooter.add(new JLabel("Http Method:"), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 15, 0, 0), 0, 0));
        ButtonGroup bg = new ButtonGroup();
        jrbGet = new JRadioButton("Get");
        bg.add(jrbGet);
        jrbGet.setSelected(true);
        jpFooter.add(jrbGet, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        jrbPost = new JRadioButton("Post");
        bg.add(jrbPost);
        jpFooter.add(jrbPost, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
        //...No of Threads
        jpFooter.add(new JLabel("No. Threads:"), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 15, 0, 0), 0, 0));
        jtfNoOfThreads = new JTextField(DEFAULT_NUMBER_OF_THREADS, 3);
        jpFooter.add(jtfNoOfThreads, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        //...No of Times
        jpFooter.add(new JLabel("No. Times:"), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 15, 0, 0), 0, 0));
        jtfNoOfTimes = new JTextField(DEFAULT_NUMBER_OF_TIMES_TO_LOOP, 3);
        jpFooter.add(jtfNoOfTimes, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        //...Latency
        jpFooter.add(new JLabel("Latency:"), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 15, 0, 0), 0, 0));
        jtfLatency = new JTextField("0", 3);
        jpFooter.add(jtfLatency, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        //...Wrap Output
        jpFooter.add(new JLabel("Wrap:"), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 15, 0, 0), 0, 0));
        jtfWrapOutput = new JTextField(""+wrapOutput, 2);
        jpFooter.add(jtfWrapOutput, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        //...Show Details
        jcbShowDetails = new JCheckBox("Show Details", showDetails);
        jpFooter.add(jcbShowDetails, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 15, 0, 0), 0, 0));
    }
    
    
    public void runTest(int numberOfTimesToRun) {
        try {
            //set the cursors
            Cursors.setWorking(Containers.getRootContainer(URLStressTester.this), true);

            //disable key fields
            jbRunTest.setEnabled(false);
            jbAddToReq.setEnabled(false);

            //get all the settings for this run
            String surl = jtfURL.getText();
            int threadCount = 1;
            try {threadCount = Integer.parseInt(jtfNoOfThreads.getText());} 
            catch (NumberFormatException e) {}
            int loopCount = 1;
            try {loopCount = Integer.parseInt(jtfNoOfTimes.getText());} 
            catch (NumberFormatException e) {}
            int latency = 0;
            try {latency = Integer.parseInt(jtfLatency.getText());} 
            catch (NumberFormatException e) {}
            wrapOutput = 60;
            try {wrapOutput = Integer.parseInt(jtfWrapOutput.getText());} 
            catch (NumberFormatException e) {}
            doGet = jrbGet.isSelected();
            isMozilla = jrbMZ.isSelected();
            showDetails = jcbShowDetails.isSelected();
            int outputCount = (showDetails ? threadCount : 1);
            thElapsed = new TreeMap();

            //create the necessary output consoles
            jpOutput.removeAll();
            jpOutput.setLayout(new GridLayout(1,outputCount,0,0));
            JTextArea jtxa[] = new JTextArea[outputCount];
            for (int i=0; i<outputCount; i++) {
                jtxa[i] = new JTextArea("");
                jtxa[i].setCursor(new Cursor(Cursor.WAIT_CURSOR));
                jtxa[i].setMinimumSize(new Dimension(20,100));
//                jtxa[i].setMaximumSize(new Dimension(100,100));
                jtxa[i].invalidate();
                JScrollPane jsp = new JScrollPane(jtxa[i]);
                jpOutput.add(jsp);
            }
            Cursors.setWorking(jpOutput, true);
//            jpOutput.invalidate();
            jspMaster.validate();
            jspMaster.paintAll(jspMaster.getGraphics());        
            Thread.yield();            
        
            //now we're ready to run the tests
            long totalMillis = 0;
            for (int loopCntr=0; loopCntr<numberOfTimesToRun; loopCntr++) {
                //create the necessary threads
                activeThreads=0;
//                threadCntr=0;
                readySetGo = false;    
                URLRequestor threads[] = new URLRequestor[threadCount];
                for (int i=0; i<threadCount; i++) {
                    int ta = 0;
                    if (i<outputCount) ta = i;
                    threads[i] = new URLRequestor(surl, i, jtxa[ta], latency*i, loopCount, loopCntr+1);
                    threads[i].start();
                }

                //sit and wait for all the threads to get up and running (but only do this if the latency is not set)
                for (int i=0; i<threadCount; i++) {
                    if (!threads[i].isRunning) {
                        --i;
                        try {
                            Thread.sleep(50);
                            Thread.yield();
                        } catch (InterruptedException ie) {}
                    }
                }

                //let er rip!
                startTime = System.currentTimeMillis();
                totalTime = 0;
                readySetGo = true;
            
                //sit and wait for all the threads to finish
                for (int i=0; i<threadCount; i++) {
                    try {threads[i].join();}
                    catch (InterruptedException ie) {}
                }        
                endTime = System.currentTimeMillis();
                println(jtxa[0], " ");
                println(jtxa[0], "Start:"+startTime);
                println(jtxa[0], "End:"+endTime, true);
//                println(jtxa[0], "Elapsed:"+(endTime-startTime)+" millisecs", true);
//                println(jtxa[0], "Average:"+((totalTime)/threadCount)+" millisecs", true);
                println(jtxa[0], "Elapsed:"+DateUtil.getElapsedStr(endTime - startTime), true);
                println(jtxa[0], "Average:"+DateUtil.getElapsedStr((totalTime)/threadCount), true);
                totalMillis += (endTime-startTime);
            }
            println(jtxa[0], " ");

            StringBuffer sb = new StringBuffer(5000);
            String sep = "";
            Iterator it = thElapsed.entrySet().iterator();
            while (it.hasNext()) {
                Map.Entry me = (Map.Entry) it.next();
                sb.append(sep+me.getValue());
                sep = ", ";
            }
            println(jtxa[0], "Elapsed values:"+sb, true);
            

            long avg = (totalMillis/numberOfTimesToRun);
//            if (numberOfTimesToRun>1) println(jtxa[0], "Average time/run over "+numberOfTimesToRun+" iterations:"+avg+" millisecs", true);
            if (numberOfTimesToRun>1) println(jtxa[0], "Average time/run over "+numberOfTimesToRun+" iterations:"+DateUtil.getElapsedStr(avg), true);
            println(jtxa[0], "Est. throughput:"+(threadCount*numberOfTimesToRun*1000/totalMillis)+" reqs/sec", true);
            println(jtxa[0], "Est. throughput:"+(threadCount*numberOfTimesToRun*1000*60/totalMillis)+" reqs/minute", true);
            println(jtxa[0], "Est. throughput:"+(threadCount*numberOfTimesToRun*1000*3600/totalMillis)+" reqs/hour", true);
        } finally {    
            //re-enable key fields
            jbRunTest.setEnabled(true);
            jbAddToReq.setEnabled(true);
            
            //reset the cursors
            Cursors.setWorking(Containers.getRootContainer(URLStressTester.this), false);
        }
    }
    
    //-------------------- URLRequestor --------------------------
    /**
     * Request a URL
     */
    class URLRequestor extends Thread {

        String surl = null;
        int threadNo = -1;
        JTextArea jtxa = null;
        int latency = -1;
        int numberOfTimes = 1;
        int currentLoopPass = -1;
        boolean isRunning = false;
            
        public URLRequestor (String iurl, int ithreadNo, JTextArea ijtxa, int ilatency, int inumberOfTimes, int icurrentLoopPass) {
            surl = iurl;
            threadNo = ithreadNo;
            jtxa = ijtxa;
            jtxa.setText("");
            latency = ilatency;
            numberOfTimes = inumberOfTimes;
            currentLoopPass = icurrentLoopPass;
            if (numberOfTimes<1) numberOfTimes = 1;
            if (showDebug>0) println(jtxa, "Creating Thread...");
        }
            
        public void run() {
            //create the HTTPRequest object
            HttpRequester hr = new HttpRequester();
            String urlStr = surl;
            String method = (doGet ? HttpRequester.GET : HttpRequester.POST);
            Map props = null;
            int spos = surl.indexOf("?");
            if (spos>0) {
                String paramStr = surl.substring(spos+1);
                urlStr = surl.substring(0,spos);
                if (showDebug>0) println(jtxa, "URL Str:"+urlStr);
                if (showDebug>0) println(jtxa, "Param Str:"+paramStr);
                props = HttpConverter.cvtURLStringToMap(paramStr, "&");
            }
            try {
                if (isMozilla) {
                    hr.addHeader("user-agent", "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.5) Gecko/20031007 Firebird/0.7");
                } else {
                    hr.addHeader("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)");
                }
                hr.setRequest(urlStr, method, props);
                
            } catch (MalformedURLException e) {
                println(jtxa, "Bad URL:"+e);
                return;
            }
            
            //...now we're officially running
            isRunning = true;

            
            //wait for the starting gun
            while (!readySetGo) {
                try {
                    Thread.sleep(10);
                    Thread.yield();
                } catch (InterruptedException ie) {}
            }
            
            //now sleep for the required amount of latency
            if (latency>0) try {
                Thread.sleep(latency);
                Thread.yield();
            } catch (InterruptedException ie) {}
            
            //Bang! Off to the races...
            long startTime = System.currentTimeMillis();
            activeThreads++;
            if (showDetails) println(jtxa, "Starting Thread "+threadNo+(latency>0 ? " (lat="+latency+")" : ""));

            //retrieve the URL
            boolean success = false;
            for (int i=0; i<numberOfTimes; i++) {
                try {
                    //make the connection
                    long bytesRead = 0;
                    long s1 = System.currentTimeMillis();
                    hr.connect();
                
                    //csc_010404_1 - added
                    //start by getting the headers
                    if (showDetails) println(jtxa, "Headers...");
                    Map hdrs = hr.getURLConnection().getHeaderFields();
                    Iterator it = hdrs.entrySet().iterator();
                    while (it.hasNext()) {
                        Map.Entry me = (Map.Entry) it.next();
                        if (showDetails) println(jtxa, "key:"+me.getKey()+" val:"+me.getValue());
                    }
            
                    //next get the data
                    if (showDetails) println(jtxa, "Data...");
                    String inputLine = null;
                    while ((inputLine = hr.readLine())!=null) {
                        bytesRead += inputLine.length();
                        if (showDetails) println(jtxa, inputLine);
                        Thread.yield();
                    }
                    hr.disconnect();
//                    if (showDetails && numberOfTimes>1) {
//                    println(jtxa, "...Finished request["+i+"], Thread "+threadNo+" read "+bytesRead+" bytes, ("+(System.currentTimeMillis()-s1)+" millis)", true);
                    println(jtxa, "...Finished request["+i+"], Thread "+threadNo+" read "+bytesRead+" bytes, "+DateUtil.getElapsedStr(System.currentTimeMillis()-s1), true);
//                        println(jtxa, "");
//                    }
                    success = true;
                } catch (Exception e) {
                    success = false;
                    println(jtxa, "Aborted Thread "+threadNo+"   "+e);
                    e.printStackTrace();
                    break;
                }
            }
            long endTime = System.currentTimeMillis();
            long elapsed = endTime-startTime;
            incrTotalTime(elapsed);
            thElapsed.put(""+currentLoopPass+","+threadNo, new Long(elapsed));
            if (success && showDetails) println(jtxa, "Finished Thread "+threadNo+" ("+(endTime-startTime)+" millis)");
        }
    }

    public void println(JTextArea jta, String s) {
        println(jta, s, logToConsole);
    }
    
    public void println(JTextArea jta, String s, boolean logToConsole) {
        for (int i=0, max=s.length(); i<max; i+=wrapOutput) {
            int epos = i+wrapOutput;
            if (epos>max) epos = max;
            if (i==0) jta.append(s.substring(i,epos)+System.getProperty("line.separator"));
            else jta.append("   "+s.substring(i,epos)+System.getProperty("line.separator"));
        }
        jta.invalidate();
        jta.paintAll(jta.getGraphics());        
        if (logToConsole) System.out.println(s);
    }
    
    public synchronized void incrTotalTime(long incr) {
        totalTime += incr;
    }
    
    

    //-------------------- ComponentListener ---------------------
    /**
     * Handle component events
     */
    class LocalComponentListener extends DefaultComponentListener {
    
        /**
         * Invoked when the component has been made visible. 
         */
        public void componentShown(ComponentEvent e) {
            if (showDebug>1) System.out.println("Shown:"+e);
            
            //init the screen and catch any unexpected errors
            if (!isInitialized) try {
                initViews();
                isInitialized = true;
            } catch (Exception be) {
                System.out.println("Error initializing "+this);
                be.printStackTrace();
            }
        }
    }

    //-------------------- ActionListener ------------------------
    /**
     * Handle the actions for buttons
     */
    class LocalActionAdapter implements java.awt.event.ActionListener {

        /**
         * actionPerformed method
         */
        public void actionPerformed(ActionEvent e) {
            //handle the action
            Object o = e.getSource();
            if (o==jbAddToReq) {
                java.awt.Toolkit.getDefaultToolkit().beep();
                JOptionPane.showMessageDialog(null, "Not implemented yet", "To Do", JOptionPane.INFORMATION_MESSAGE);
            } else if (o==jbRunTest) {
                runTest(1);
            } else if (o==jbAvgTest) {
                runTest(10);
            } 
        }
    }
    
}
