package org.barracudamvc.tutorials.comp;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BTemplate;
import org.barracudamvc.core.comp.DefaultTemplateView;
import org.barracudamvc.core.comp.IterativeModel;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.helper.ComponentGateway;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.tutorials.xmlc.HelloWorld2b1HTML;
import org.w3c.dom.Document;

/**
 * Here's an example of HelloWorld using the BTemplate
 * component
 */
public class HelloWorld2b extends ComponentGateway {

    //xmlc factory (ok as static because its threadsafe)
//     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //figure out which page we're after by looking for an optional
        //page parameter in the Req
        String pageNo = req.getParameter("page");
        if (pageNo==null) pageNo = "1";
        Class pageCl = null;
        try {
            pageCl = Class.forName("org.barracudamvc.tutorials.xmlc.HelloWorld2b"+pageNo+"HTML", true, Thread.currentThread().getContextClassLoader());
        } catch (ClassNotFoundException e) {
            pageNo = "1";
            pageCl = HelloWorld2b1HTML.class;
        }

        //load the DOM object
//        if (xmlcFactory==null) xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
//        XMLObject page = xmlcFactory.create(pageCl);

        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(pageCl, vc.getViewCapabilities().getClientLocale());

        //create the component and bind it to the view
        BTemplate templateComp = new BTemplate();
        templateComp.setView(new DefaultTemplateView(page.getElementById("HelloWorld")));

        //now specify the model
        templateComp.addModel(new HelloWorldModel(pageNo));
        String[] items = new String[] {"Cabbage", "Carrots", "Pickles", "Sugar", "Wheaties", "Flour", "Potatoes", "Frozen Pepperoni Pizzas", "Pineapples", "Honey-blanched Sardines", "Adobo", "Fresh Garlic cloves", "Milk", "Chardonnay", "Spam", "Pickled Pigs Feet", "Antacid"};
        String[] qtys =  new String[] {"3 heads", "1 bag", "1 jar", "5 lbs", "3 bxs", "50 lbs", "25 lbs", "17", "3", "6 cans", "2 jars", "20", "5 gallons", "3 bottles", "1 crate", "2 jars", "1 bottle"};
        templateComp.addModel(new GroceriesModel(items, qtys));
        
        //add the component to the root
        root.addChild(templateComp);
    
        //return the page
        return page;
    }

    //--------------- GroceriesModel -------------------------
    /**
     * Groceries Model
     */
    class GroceriesModel extends AbstractTemplateModel implements IterativeModel {

        String[] items = null;
        String[] qtys = null;
        int cntr = -1;

        //constructor
        public GroceriesModel(String[] iitems, String[] iqtys) {
            items = iitems;
            qtys = iqtys;
        }

        //register the model by name
        public String getName() {return "Groceries";}

        //provide items by key
        public Object getItem(String key) {
            if (key.equals("Item")) return items[cntr];
            else if (key.equals("Qty")) return qtys[cntr];
            else if (key.equals("RowCntr")) return ""+(cntr+1);
            else return super.getItem(key);
        }

        //prepare for iteration
        public void preIterate() {
            cntr = -1;
        }
        
        //return true if there are more rows in the data model
        public boolean hasNext() {
            return (cntr<(items.length-1));
        }
        
        //load the next row in the model
        public void loadNext() {
            cntr++;
        }

        //cleanup after iteration
        public void postIterate() {}
    }

    //--------------- HelloWorldModel ------------------------
    /**
     * HelloWorldModel
     */
    class HelloWorldModel extends AbstractTemplateModel {
        String pageNo = null;
    
        //constructor
        public HelloWorldModel(String ipageNo) {
            pageNo = ipageNo;
        }

        //register the model by name
        public String getName() {
            return "HelloWorld";
        }
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("Title")) return "Hello World 2b example (ex "+pageNo+")";
            else if (key.equals("Descr")) return "This example illustrates how using BTemplate allows us to drastically alter the look and feel of a page just by modifying the template; the developer doesn't need to make any changes to the components or the models in any of these examples. As you can see, this is quite flexible (we can shift between lists, tables, etc).";
            else if (key.equals("Hello")) return "Hello World! Hi Ma! Hi Pa!";
            else if (key.equals("Summary")) return "Shopping List Table";
            else if (key.equals("Header")) return HelloWorld2bHeader.getNode(vc);
            else if (key.equals("Footer")) return HelloWorld2Footer.getNode(vc);
            else return super.getItem(key);
        }
    }
}
