/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestFormatType.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.view;

import junit.framework.*;

import org.apache.log4j.*;
import org.barracudamvc.testbed.*;


/**
 * This test verifies that the hierachical organization of 
 * FormatType classes. We do this by testing for what it should be 
 * an instanceof and what it shouldn't. The idea is that the FormatType
 * extension hierarchy is very important and we don't want it getting 
 * changed accidentally. This test should preclude that.
 */
public class TestFormatType extends DefaultTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestFormatType.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    FormatType ft = null;
    Class targetCl = null;
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestFormatType(String name) {
        super(name);
    }

    public TestFormatType(String name, FormatType ift, Class itargetCl) {
        super(name);
        ft = ift;
        targetCl = itargetCl;
    }
    
    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    /**
     * Return a suite of tests...
     */
	public static Test suite() {
        //create the test suite
		TestSuite suite = new TestSuite();
        
        //add the tests
        //...HTML_3_0
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_0, FormatType.UnknownFormat.class));
        //...HTML_3_1
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_1, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_1, FormatType.UnknownFormat.class));
        //...HTML_3_2
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_3_2, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_3_2, FormatType.UnknownFormat.class));
        //...HTML_4_0
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_0, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_0, FormatType.UnknownFormat.class));
        //...HTML_4_1
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.Html4x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.Html3x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.Html.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.HTML_4_1, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_1, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_1, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_1, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.HTML_4_1, FormatType.UnknownFormat.class));
        //...CHTML_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.Chtml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.Chtml2x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.Html.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_1_0, FormatType.UnknownFormat.class));
        //...CHTML_2_0
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.Chtml2x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.Chtml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.Chtml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.CHTML_2_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_2_0, FormatType.Html.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_2_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_2_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.CHTML_2_0, FormatType.UnknownFormat.class));
        //...XML_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.XML_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XML_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XML_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XML_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XML_1_0, FormatType.UnknownFormat.class));
        //...VXML_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.Vxml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.VXML_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.VXML_1_0, FormatType.UnknownFormat.class));
        //...WML_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.Wml1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.WML_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.WML_1_0, FormatType.UnknownFormat.class));
        //...XHTML_BASIC_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.XhtmlBasic1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.XhtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_BASIC_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.XhtmlBasic2x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.XhtmlStandard.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_BASIC_1_0, FormatType.UnknownFormat.class));
        //...XHTML_STANDARD_1_0
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlStandard1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlStandard.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlBasic1x.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlBasic.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Xhtml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Xml.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.XHTML_STANDARD_1_0, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlStandard2x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.XhtmlBasic2x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Xml1x.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Vxml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.Wml.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.HtmlBasic.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.BinaryData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.XHTML_STANDARD_1_0, FormatType.UnknownFormat.class));
        //...UNKNOWN_FORMAT
        suite.addTest(new TestFormatType("testInstance", FormatType.UNKNOWN_FORMAT, FormatType.UnknownFormat.class));
        suite.addTest(new TestFormatType("testInstance", FormatType.UNKNOWN_FORMAT, FormatType.GenericData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.UNKNOWN_FORMAT, FormatType.AsciiData.class));
        suite.addTest(new TestFormatType("testNotInstance", FormatType.UNKNOWN_FORMAT, FormatType.BinaryData.class));

        //finally return the suite
        return suite;
    }


    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Make sure that the class being tested IS an instance of the 
     * target class
     */
    public void testInstance() {
        if (logger.isInfoEnabled()) logger.info("testing instanceof");
        assertTrue(ft+" not an instanceof "+targetCl, targetCl.isAssignableFrom(ft.getClass()));
    }

    /**
     * Make sure that the class being tested is NOT an instance of the 
     * target class
     */
    public void testNotInstance() {
        if (logger.isInfoEnabled()) logger.info("testing !instanceof");
        assertTrue(ft+" is an instanceof "+targetCl+" (and it shouldn't be!)", !targetCl.isAssignableFrom(ft.getClass()));
    }
}
