/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestStringUtil.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.plankton;

import junit.framework.*;
import org.apache.log4j.*;
import org.barracudamvc.testbed.*;


/**
 * Test the StringUtil class
 */
public class TestStringUtil extends DefaultTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestStringUtil.class.getName();
    private static final Logger logger = Logger.getLogger("test."+testClass);

    //variables
    protected String sourceStr = null;
    protected String oldPattern = null;
    protected String newPattern = null;
    protected String expectedResult = null;

    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestStringUtil(String name) {
        super(name);
//logger.setPriority(org.apache.log4j.Priority.DEBUG);
//com.atmr.kilimanjaro.nxgenw.sql.RepSQL.logger.setPriority(org.apache.log4j.Priority.DEBUG);
    }

    /**
     * Public Constructor
     */
    public TestStringUtil(String name, String isourceStr, String ioldPattern, String inewPattern, String iexpectedResult) {
        super(name);
        sourceStr = isourceStr;
        oldPattern = ioldPattern;
        newPattern = inewPattern;
        expectedResult = iexpectedResult;
    }

    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    /**
     * Return a suite of tests...
     */
    public static Test suite() {
        //create the test suite
        TestSuite suite = new TestSuite();

        //add the tests
        suite.addTest(new TestStringUtil("testReplace", null, null, "Blah", "Blah"));
        suite.addTest(new TestStringUtil("testReplace", null, "a", "Blah", null));
        suite.addTest(new TestStringUtil("testReplace", "Foo Blah", null, "_", "Foo Blah"));
        suite.addTest(new TestStringUtil("testReplace", "Foo Blah", " ", "_", "Foo_Blah"));
        suite.addTest(new TestStringUtil("testReplace", "Foo Blah Blah", " ", "_", "Foo_Blah_Blah"));
        suite.addTest(new TestStringUtil("testReplace", "Foo  Blah", " ", "_", "Foo__Blah"));
        suite.addTest(new TestStringUtil("testReplace", "~Some Text", "~", null, "Some Text"));
        suite.addTest(new TestStringUtil("testReplace", "Some Text~", "~", null, "Some Text"));
        suite.addTest(new TestStringUtil("testReplace", "~Some~Text~", "~", null, "SomeText"));
        suite.addTest(new TestStringUtil("testReplace", "~~~", "~", null, null));
        suite.addTest(new TestStringUtil("testReplace", "~~~", "~", "", ""));
        suite.addTest(new TestStringUtil("testReplace", "", " ", null, ""));

        //finally return the suite
        return suite;
    }

    //-------------------- Actual Tests --------------------------
    /**
     * Test a table to make sure it contains the expected columns
     */
    // jrk_2002-12-01.2 - the sql libraries are unused.  No sense in referencing them.
    //public void testReplace() throws SQLException {
    public void testReplace() {
        logger.info("running testReplace()");

        String actualResult = StringUtil.replace(sourceStr, oldPattern, newPattern);
        assertTrue("Replace err on source:'"+sourceStr+"' ('"+oldPattern+"'-->'"+newPattern+"') got:'"+actualResult+"' expected:'"+expectedResult+"'", actualResult==expectedResult || (actualResult!=null && actualResult.equals(expectedResult)));
    }
}
