/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestLocalizeFileNames.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.taskdefs;

import java.io.IOException;

import org.enhydra.xml.xmlc.taskdef.*;

import org.barracudamvc.testbed.*;

/**
 * Unit tests for buildClassName() method of LocalizeXmlcUtilsImpl
 */
public class TestLocalizeFileNames extends DefaultTestCase {

  //===========================================================================
  // CONSTRUCTORS
  //===========================================================================

    /**
     * Default constructor
     *
     * @param theName   The name of the test
     */
    public TestLocalizeFileNames( String theName) {
        super( theName );
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     */
    protected void setUp() throws Exception {
        super.setUp();
        // save default xmlc utils class
        this.myOriginalDefautlUtilsClass =
          System.getProperty( XmlcUtils.DEFAULT_UTILS_CLASS_KEY, "" );
        System.setProperty(XmlcUtils.DEFAULT_UTILS_CLASS_KEY,
                       LocalizeXmlcUtilsImpl.class.getName());
    }

    /**
     * Tears down the fixture, for example, close a network connection.
     * This method is called after a test is executed.
     */
    protected void tearDown() throws Exception {
        super.tearDown();

        // restore default xmlc utils class
        System.setProperty(XmlcUtils.DEFAULT_UTILS_CLASS_KEY,
                       this.myOriginalDefautlUtilsClass);
    }

    //===========================================================================
    // TEST METHODS
    //===========================================================================
    public void testBuildClassName() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();
        assertEquals( "Should have localize implementation",
                      LocalizeXmlcUtilsImpl.class,
                      xmlcUtils.getClass() );

        String filename = "config";
        String typeModifier = "HTML";
        String expectedClassName = "configHTML";
        String newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 0 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "config_aa";
        expectedClassName = "configHTML_aa";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 1 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "config_test";
        expectedClassName = "config_testHTML";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 2 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "config_eng";
        expectedClassName = "config_engHTML";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 3 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "config_aaab";
        expectedClassName = "config_aaabHTML";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 4 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "name1/name2/name1/name2/config_zu";
        expectedClassName = "name1.name2.name1.name2.configHTML_zu";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 5 should be as expected",
                     expectedClassName,
                     newClassName );
    }


    //===========================================================================
    // DATA MEMBERS
    //===========================================================================
    private String myOriginalDefautlUtilsClass;

}
